#include <ESP8266WiFi.h>
#include <LedControl.h>
#include <Vector.h>

int DIN = 16; // D0

int CS =  5;  // D1

int CLK = 4;  // D2

const char* ssid     = "--";
const char* password = "--";

LedControl lc=LedControl(DIN,CLK,CS,0);

WiFiServer server(80);

String header;
String page;

unsigned long currentTime = millis();

unsigned long previousTime = 0; 

const long timeoutTime = 1000;

const int UP = 0;
const int LEFT = 1;
const int RIGHT = 2;
const int DOWN = 3;

const int MAX_SIZE = 15;

int direction = -1;

int x = 4;
int y = 5;
int x_change = 0;
int y_change = 0;

const int ELEMENT_COUNT_MAX = 30;
int storage_array[ELEMENT_COUNT_MAX];
Vector<int> snake(storage_array);
int len = 1;

int fruitX = 3;
int fruitY = 2;

boolean game_over = false;

void setup() {
  Serial.begin(9600);
  lc.shutdown(0,false);     
  lc.setIntensity(0,8);
  lc.clearDisplay(0);        
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  server.begin();

}


void loop(){
  delay(1000);
  Serial.println(direction);
  x += x_change;
  y += y_change;
  lc.setLed(0, fruitX, fruitY, true);
  snake.push_back(x);
  snake.push_back(y);
  if (snake.size() / 2 > len) {
    snake.remove(0);
    snake.remove(0);
  }
  for (int i = 0; i < snake.size(); i += 2) {
    Serial.println(snake[i]);
    Serial.println(snake[i + 1]);
  }
  our_snake(snake);

  if (x == fruitX && y == fruitY) {
    makeFruit();
    len++;
  }

  if (x > 7 || x < 0 || y > 7 || y < 0) {
    x = 4;
    y = 5;
    fruitX = 3;
    fruitY = 2;
    len = 1;
    while (snake.size() != 0) {
      snake.pop_back();
    }
  }
  

  WiFiClient client = server.available();   

  if (client) {                             
    Serial.println("New Client.");          
    String currentLine = "";                
    currentTime = millis();
    previousTime = currentTime;
    while (client.connected() && currentTime - previousTime <= timeoutTime) { 
      currentTime = millis();         
      if (client.available()) {             
        char c = client.read();             
        Serial.write(c);                    
        header += c;
        if (c == '\n') {                    
          if (currentLine.length() == 0) {
   
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();


            if(header.indexOf("GET /") >= 0) {
              direction = -1;
            }

            if (header.indexOf("GET /1") >= 0) {
              direction = UP;
              y_change = 0;
              x_change--;
              
            } else if(header.indexOf("GET /4") >= 0) {
              direction = DOWN;
              y_change = 0;
              x_change++;
              
            } else if(header.indexOf("GET /2") >= 0) {
              direction = LEFT;
              x_change = 0;
              y_change--;
              
            } else if(header.indexOf("GET /3") >= 0) {
              direction = RIGHT;
              x_change = 0;
              y_change++;
            }
          
            
            
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #195B6A; border: none; color: white; padding: 16px 40px;");
            client.println("text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}");
            client.println(".button2 {background-color: #77878A;}</style></head>");
            
            client.println("<body><h1>Web Snake</h1>");
            client.println("<button class=\"button\"> <a href=\"/1\"> UP </a> </button>");
            client.println("<br>");
            client.println("<button class=\"button\"> <a href=\"/2\"> LEFT </a> </button>");
            client.println("<button class=\"button\"> <a href=\"/3\"> RIGHT </a> </button>");
            client.println("<br>");
            client.println("<button class=\"button\"> <a href=\"/4\"> DOWN </a> </button>");
            
            client.println("</body></html>");
            
            client.println();
            break;
          } else { 
            currentLine = "";
          }
        } else if (c != '\r') {  
          currentLine += c;      
        }
      }
    }
   
    header = "";
   
    client.stop();
    Serial.println("Client disconnected.");
    Serial.println("");
  }
}

void makeFruit(){
  int x, y;
  x = random(0, 8);
  y = random(0, 8);
  while(isPartOfSnake(x, y)){
    x = random(0, 8);
    y = random(0, 8);
  }
  fruitX = x;
  fruitY = y;
}

boolean isPartOfSnake(int x, int y){
  for(int i = 0; i < snake.size(); i+=2)
  {
    if((x == snake[i]) && (y == snake[i + 1]))
      return true;
  }
  return false;
}

void our_snake(Vector<int> snake) {
  for (int i = 0; i < 8; i++) {
    for (int j = 0; j < 8; j++) {
       lc.setLed(0, i, j, false);
       lc.setLed(0, fruitX, fruitY, true);
    }
  }
  for (int i = 0; i < snake.size(); i += 2) {
    lc.setLed(0, snake[i], snake[i + 1], true);
  }
}
